export type IStreamCallback<T> = (
   msg: T | undefined,
   error: Error | undefined,
   end: boolean
) => void;

export interface IStream<T> {
   subscribe(callback: IStreamCallback<T>): void;
   close(): void;
}

export class Stream<T> implements IStream<T> {
   buffer: [T | undefined, Error | undefined, boolean][] = [];
   closed = false;
   subscriber: IStreamCallback<T>[] = [];

   subscribe(cb: IStreamCallback<T>) {
      if (this.buffer.length > 0) {
         this.buffer.forEach((e) => cb(...e));
         this.buffer = [];
      }
      if (!this.closed) this.subscriber.push(cb);
   }

   send(msg: T) {
      if (this.subscriber.length <= 0) {
         this.buffer.push([msg, undefined, false]);
      }

      this.subscriber.forEach((s) => s(msg, undefined, false));
   }

   error(error: Error) {
      if (this.subscriber.length <= 0) {
         this.buffer.push([undefined, error, false]);
      }
      this.subscriber.forEach((s) => s(undefined, error, false));
   }

   close() {
      this.closed = true;
      if (this.subscriber.length <= 0) {
         this.buffer.push([undefined, undefined, true]);
      }
      this.subscriber.forEach((s) => s(undefined, undefined, true));
      this.subscriber = []; // Clear after close
   }
}
