import SyncInitRequest, { serialize_SyncInitRequest, deserialize_SyncInitRequest} from "./syncinitrequest";
import SyncInitResponse, { serialize_SyncInitResponse, deserialize_SyncInitResponse} from "./syncinitresponse";
import PullGetLogRequest, { serialize_PullGetLogRequest, deserialize_PullGetLogRequest} from "./pullgetlogrequest";
import PullGetLogResponse, { serialize_PullGetLogResponse, deserialize_PullGetLogResponse} from "./pullgetlogresponse";
import PullObjectRequest, { serialize_PullObjectRequest, deserialize_PullObjectRequest} from "./pullobjectrequest";
import PullObjectResponse, { serialize_PullObjectResponse, deserialize_PullObjectResponse} from "./pullobjectresponse";
import PushObjectRequest, { serialize_PushObjectRequest, deserialize_PushObjectRequest} from "./pushobjectrequest";
import PushObjectResponse, { serialize_PushObjectResponse, deserialize_PushObjectResponse} from "./pushobjectresponse";
import ServiceMessage, { serialize_ServiceMessage, deserialize_ServiceMessage} from "./servicemessage";
export type {
   SyncInitRequest,
   SyncInitResponse,
   PullGetLogRequest,
   PullGetLogResponse,
   PullObjectRequest,
   PullObjectResponse,
   PushObjectRequest,
   PushObjectResponse,
   ServiceMessage,
}
import { IStream, Stream, Service, ServiceProvider, getRandomID } from "./service_client";
export class Sync extends Service {
   constructor(provider: ServiceProvider){
      super(provider, "Sync");
      this.functions.set("SyncInit", {
         istream: false, input: deserialize_SyncInitRequest,
         rstream: false, ret: serialize_SyncInitResponse
      });
      this.functions.set("PullGetLog", {
         istream: false, input: deserialize_PullGetLogRequest,
         rstream: false, ret: serialize_PullGetLogResponse
      });
      this.functions.set("PullObject", {
         istream: false, input: deserialize_PullObjectRequest,
         rstream: false, ret: serialize_PullObjectResponse
      });
      this.functions.set("PushObject", {
         istream: false, input: deserialize_PushObjectRequest,
         rstream: false, ret: serialize_PushObjectResponse
      });
   }

   SyncInit(data: SyncInitRequest): Promise<SyncInitResponse> {
      const msgid = getRandomID(16);
      return new Promise((resolve, reject)=>{
         this.calls.set(msgid, (msg)=>{
            this.calls.delete(msgid);
            if(msg.error) reject(new Error(msg.error))
            else resolve(deserialize_SyncInitResponse(msg.payload))
         })
         this.sendMessage({
            service: "Sync",
            function: "SyncInit",
            id: msgid,
            payload: (serialize_SyncInitRequest(data)) as any,
            error: undefined as any
         });
      })
      
   }

   PullGetLog(data: PullGetLogRequest): Promise<PullGetLogResponse> {
      const msgid = getRandomID(16);
      return new Promise((resolve, reject)=>{
         this.calls.set(msgid, (msg)=>{
            this.calls.delete(msgid);
            if(msg.error) reject(new Error(msg.error))
            else resolve(deserialize_PullGetLogResponse(msg.payload))
         })
         this.sendMessage({
            service: "Sync",
            function: "PullGetLog",
            id: msgid,
            payload: (serialize_PullGetLogRequest(data)) as any,
            error: undefined as any
         });
      })
      
   }

   PullObject(data: PullObjectRequest): Promise<PullObjectResponse> {
      const msgid = getRandomID(16);
      return new Promise((resolve, reject)=>{
         this.calls.set(msgid, (msg)=>{
            this.calls.delete(msgid);
            if(msg.error) reject(new Error(msg.error))
            else resolve(deserialize_PullObjectResponse(msg.payload))
         })
         this.sendMessage({
            service: "Sync",
            function: "PullObject",
            id: msgid,
            payload: (serialize_PullObjectRequest(data)) as any,
            error: undefined as any
         });
      })
      
   }

   PushObject(data: PushObjectRequest): Promise<PushObjectResponse> {
      const msgid = getRandomID(16);
      return new Promise((resolve, reject)=>{
         this.calls.set(msgid, (msg)=>{
            this.calls.delete(msgid);
            if(msg.error) reject(new Error(msg.error))
            else resolve(deserialize_PushObjectResponse(msg.payload))
         })
         this.sendMessage({
            service: "Sync",
            function: "PushObject",
            id: msgid,
            payload: (serialize_PushObjectRequest(data)) as any,
            error: undefined as any
         });
      })
      
   }
}